﻿using System;
using System.Net; // für WebProxy
using System.Net.Http;
using Serilog;

namespace Datev.Cloud.Authentication.Sample
{
    class Program
    {
        static void Main()
        {

            //Logging auf der Console aktivieren
            Log.Logger = new LoggerConfiguration()
                             .MinimumLevel.Information()
                             .WriteTo.Console()
                             .CreateLogger();

            //diese ClientId kommt nur auf den Funktionstest
            var options = new ClientOptions("https://login.datev.de/openidsandbox", "e8acebcf-eeb8-413e-9db3-e1ddd667649b", "9US8!CbULcxp#&ja5QaX", "openid functiontest", 51234);

            var configOptions = new ClientConfigOptions
            {
                //RequireAuthenticationMethod = AuthenticationMethod.SmartCard,  // Anmeldung mit SmartCard erzwingen 
                //RequireAuthenticationMethod = AuthenticationMethod.SmartLogin // Anmeldung mit SmartLogin erzwingen
            };

            // Client anlegen ohne http proxy
            var client = new Client(options, configOptions);

            //// Client anlegen mit http Proxy 
            //var cred = new NetworkCredential("User", "Passwort");
            //var proxy = new WebProxy("proxy:8080", false, new string[] { }, cred);
            //var client = new Client(options,configOptions, proxy);

            // Login
            var result = client.LoginAsync().Result;
            if (result.Success)
            {
                Console.WriteLine("Login ok \r\nAccessToken: " + result.AccessToken);

                // RefreshTokenHandler für API Client ohne http Proxy 
                var refreshTokenHandler = new RefreshTokenHandler(client, result.RefreshToken, result.AccessToken);

                //// RefreshTokenHandler für API Client mit http Proxy
                //var proxyHandler = new HttpClientHandler
                //{
                //    Proxy = proxy,
                //    UseProxy = true,
                //};
                //var refreshTokenHandler = new RefreshTokenHandler(client, result.RefreshToken, result.AccessToken,proxyHandler);

                // http Client erzeugen mit Handler der alle Tokens und Logik zum refesh der Tokens enthält
                // funktioniert default 11 Stunden lang, wenn die Anwendung länger läuft muss man entweder:
                // nach 11 Stunden einen neue Anmeldung machen mit client.LoginAsync()
                // oder auf http Statuscode 401 reagieren und daraufhin eine neue Anmeldung durchführen.
                var apiClient = new HttpClient(refreshTokenHandler);

                // API Call
                var apiResult = apiClient.GetAsync("https://sandbox-api.datev.de/functiontest/v1/identity").Result;

                // Ergebnis auswerten
                if (apiResult.IsSuccessStatusCode)
                {
                    Console.WriteLine(" Api Call ok, Content: " + apiResult?.Content?.ReadAsStringAsync().Result);
                }
                else
                {
                    Console.WriteLine(" Api Call Error: " + apiResult.StatusCode + " Content:" + apiResult?.Content?.ReadAsStringAsync().Result);
                }

                // den apiClient nach dem Request weiter verwenden für alle weiteren API Calls.
            }
            else
            {
                Console.WriteLine("Login Error: " + result.Error);
            }
        }
    }
}

